@@ -647,9 +647,14 @@ Rcpp::List MAR_mv(arma::mat data,
   int start_s = clock();
   int current_s;
 
+  // TEMP TEMP TEMP TEMP
+  arma::mat times_in(niter, 6);
+
   // strarting loop
   for(arma::uword iter = 0; iter < niter; iter++){
 
+    int in_start_s = clock();
+
     // accelerating
     accelerate_MAR_mv(data,
                       mu,
@@ -659,6 +664,8 @@ Rcpp::List MAR_mv(arma::mat data,
                       k0,
                       S0,
                       n0);
+    times_in(iter, 0) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     if(hyper){
       hyper_accelerate_MAR_mv_LS(mu,
@@ -674,6 +681,8 @@ Rcpp::List MAR_mv(arma::mat data,
                                  theta1,
                                  Theta1);
     }
+    times_in(iter, 1) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     // update cluster allocation
     clust_update_MAR_PY_mv(data,
@@ -691,6 +700,8 @@ Rcpp::List MAR_mv(arma::mat data,
     para_clean_MAR_mv(mu,
                       s2,
                       clust);
+    times_in(iter, 2) = double(clock() - in_start_s ) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     // if the burn-in phase is complete
     if(iter >= nburn){
@@ -749,6 +760,8 @@ Rcpp::List MAR_mv(arma::mat data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
+  // TEMP TEMP TEMP
+  resu["wvals"] = times_in;
   return resu;
 }
 
@@ -1169,3 +1182,210 @@ Rcpp::List MAR_mv_MKR(arma::vec y,
   }
   return resu;
 }
+
+/*----------------------------------------------------------------------
+ *
+ * MULTIVARIATE
+ * MIXTURE OF KERNEL REGRESSION - LOCATION
+ * importance conditional sampler function
+ *
+ *----------------------------------------------------------------------
+ */
+
+//' @export
+//' @name MAR_mv_MKR_L
+//' @title C++ function to estimate Pitman-Yor multivariate mixtures via marginal sampler - PRODUCT KERNEL
+//' @keywords internal
+//'
+//' @param data a matrix of observations
+//' @param grid matrix of points to evaluate the density
+//' @param niter number of iterations
+//' @param nburn number of burn-in iterations
+//' @param m0 expectation of location component
+//' @param k0 vector, scale parameters for the location component
+//' @param a0 vector, parameters of scale component
+//' @param b0 vector, parameters of scale component
+//' @param m1 means of hyperdistribution of m0
+//' @param s21 variances of hyperdistribution of m0
+//' @param a1 shape parameters of hyperdistribution of b0
+//' @param b1 rate parameters of hyperdistribution of b0
+//' @param strength strength parameter
+//' @param napprox number of approximating values
+//' @param nupd number of iterations to show current updating
+//' @param out_param if TRUE, return also the location and scale paramteres lists
+//' @param out_dens if TRUE, return also the estimated density (default TRUE)
+//' @param discount second parameter of PY
+//' @param print_message print the status
+//' @param light_dens if TRUE return only the posterior mean of the density
+//' @param hyper, if TRUE use hyperpriors, default TRUE
+//
+
+//[[Rcpp::export]]
+Rcpp::List MAR_mv_MKR_L(arma::vec y,
+                        arma::mat covs,
+                        arma::vec grid_response,
+                        arma::mat grid_covs,
+                        int niter,
+                        int nburn,
+                        arma::vec beta0,
+                        arma::mat Sb0,
+                        double a0,
+                        double b0,
+                        arma::vec beta1,
+                        double k1,
+                        double sb1,
+                        arma::mat Sb1,
+                        double strength,
+                        int napprox,
+                        int nupd = 0,
+                        bool out_param = 0,
+                        bool out_dens = 1,
+                        double discount = 0,
+                        bool print_message = 1,
+                        bool light_dens = 1,
+                        bool hyper = 1){
+
+  if(nupd == 0){
+    nupd = (int) (niter / 10);
+  }
+  int n = covs.n_rows;
+  int d = covs.n_cols;
+
+  // initialize results objects
+  arma::mat result_clust(niter - nburn, n);
+  std::list<arma::mat> result_beta;
+  arma::vec result_sigma2(niter - nburn);
+  std::list<arma::vec> result_probs;
+
+  // initialize the result densities objects (response and covs)
+  arma::cube result_dens(grid_response.n_rows, grid_covs.n_rows, 1);
+  if(!light_dens){
+    result_dens.resize(grid_response.n_rows, grid_covs.n_rows, niter - nburn);
+  }
+  result_dens.fill(0);
+
+  // initialize required object inside the loop
+  arma::vec clust(n);
+  arma::mat beta(1,d);
+  double sigma2;
+  arma::mat dens(grid_response.n_rows, grid_covs.n_rows);
+
+  // fill the initialized quantity
+  clust.fill(0);
+  beta.row(0) = arma::trans(beta0);
+  sigma2 = b0 / (a0 - 1);
+
+  // time quantities
+  int start_s = clock();
+  int current_s;
+
+  // strarting loop
+  for(arma::uword iter = 0; iter < niter; iter++){
+
+    // acceleration step
+    accelerate_MAR_mv_MRK_L(y,
+                            covs,
+                            beta,
+                            sigma2,
+                            clust,
+                            beta0,
+                            Sb0,
+                            a0,
+                            b0);
+
+    if(hyper){
+      hyper_accelerate_MAR_mv_MRK_L(y,
+                                    covs,
+                                    clust,
+                                    beta,
+                                    beta0,
+                                    Sb0,
+                                    a0,
+                                    b0,
+                                    beta1,
+                                    k1,
+                                    sb1,
+                                    Sb1);
+
+    }
+
+    // update cluster allocations
+    clust_update_MAR_mv_MRK_L(y,
+                              covs,
+                              beta,
+                              sigma2,
+                              clust,
+                              strength,
+                              beta0,
+                              Sb0,
+                              discount,
+                              napprox);
+
+    // clean the parameters
+    para_clean_MAR_mv_MRK_L(beta,
+                            clust);
+
+    // if the burn-in phase is complete
+    if(iter >= nburn){
+      arma::vec tab_freq = freq_vec(clust) - discount;
+      result_probs.push_back(tab_freq / n);
+
+      result_beta.push_back(beta);
+      result_sigma2(iter - nburn) = sigma2;
+      result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+
+      if(out_dens){
+        dens = eval_density_mv_MKR_L(grid_covs,
+                                     grid_response,
+                                     beta,
+                                     sigma2,
+                                     tab_freq);
+
+        if(light_dens){
+          result_dens.slice(0) += dens;
+        } else {
+          result_dens.slice(iter - nburn) = dens;
+        }
+      }
+
+    }
+
+    if(print_message){
+      // print the current completed work
+      if((iter + 1) % nupd == 0){
+        current_s = clock();
+        Rcpp::Rcout << "Completed:\t" << (iter + 1) << "/" << niter << " - in " <<
+          double(current_s-start_s)/CLOCKS_PER_SEC << " sec\n";
+      }
+    }
+    Rcpp::checkUserInterrupt();
+  }
+
+  int end_s = clock();
+  if(print_message){
+    Rcpp::Rcout << "\n" << "Estimation done in " << double(end_s-start_s)/CLOCKS_PER_SEC << " seconds\n";
+  }
+
+  Rcpp::List resu;
+  if(out_param){
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["beta"]   = result_beta;
+    resu["sigma2"] = result_sigma2;
+    resu["probs"]  = result_probs;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  } else {
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  }
+  return resu;
+}
