@@ -299,6 +299,13 @@ Rcpp::List cICS(arma::vec data,
   arma::mat result_dens(niter - nburn, grid.n_elem);
   result_dens.fill(0);
 
+  ////////////////////////////////////////////////
+  ////////////////////////////////////////////////
+  // TEMP
+  arma::mat result_dens_dev(niter - nburn, grid.n_elem);
+  ////////////////////////////////////////////////
+  ////////////////////////////////////////////////
+
   // initialize required object inside the loop
   arma::vec clust(n);
   arma::vec mu(1);
@@ -410,6 +417,23 @@ Rcpp::List cICS(arma::vec data,
       result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
     }
 
+    ///////////////////////////////////////////////////////////////////////
+    ///////////////////////////////////////////////////////////////////////
+    // temp////////////////////////////////////////////////////////////////
+    if(iter >= nburn){
+      arma::vec tab_freq = freq_vec(clust) - discount;
+      if(out_dens){
+        dens = eval_density(grid,
+                            mu,
+                            s2,
+                            tab_freq);
+        result_dens_dev.row(iter - nburn) = arma::trans(dens);
+      }
+    }
+    ////////////////////////////////////////////////////////////////////////
+    ////////////////////////////////////////////////////////////////////////
+    ////////////////////////////////////////////////////////////////////////
+
     if(print_message){
       // print the current completed work
       if((iter + 1) % nupd == 0){
@@ -425,7 +449,7 @@ Rcpp::List cICS(arma::vec data,
     Rcpp::Rcout << "\n" << "Estimation done in " << double(end_s-start_s)/CLOCKS_PER_SEC << " seconds\n";
   }
 
-  Rcpp::List resu;
+    Rcpp::List resu;
   if(out_param){
     resu["dens"]   = result_dens;
     resu["clust"]  = result_clust;
@@ -438,6 +462,15 @@ Rcpp::List cICS(arma::vec data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
+
+  ///////////////////////////////////////////////////////////////////////
+  ///////////////////////////////////////////////////////////////////////
+  // temp////////////////////////////////////////////////////////////////
+  resu["tdns"] = result_dens_dev;
+  ///////////////////////////////////////////////////////////////////////
+  ///////////////////////////////////////////////////////////////////////
+  ///////////////////////////////////////////////////////////////////////
+
   return resu;
 }
 
@@ -764,9 +797,13 @@ Rcpp::List cICS_mv(arma::mat data,
   int start_s = clock();
   int current_s;
 
+  // TEMP TEMP TEMP TEMP
+  arma::mat times_in(niter, 6);
   // strarting loop
   for(arma::uword iter = 0; iter < niter; iter++){
 
+    int in_start_s = clock();
+
     // acceleration step
     accelerate_ICS_mv(data,
                       mu,
@@ -776,6 +813,8 @@ Rcpp::List cICS_mv(arma::mat data,
                       k0,
                       S0,
                       n0);
+    times_in(iter, 0) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     if(hyper){
       hyper_accelerate_ICS_mv_LS(mu,
@@ -791,12 +830,17 @@ Rcpp::List cICS_mv(arma::mat data,
                                  theta1,
                                  Theta1);
     }
+    times_in(iter, 1) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     // sample the probability vector
     // from Dirichlet distribution
     arma::vec temp_freq = freq_vec(clust);
     ptilde = rdirich_mass(temp_freq - discount, strength + discount * temp_freq.n_elem);
 
+    times_in(iter, 2) = double(clock() - in_start_s ) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
+
     // simulate the required values
     simu_trunc_PY_mv(mutemp,
                      s2temp,
@@ -808,6 +852,7 @@ Rcpp::List cICS_mv(arma::mat data,
                      n0,
                      napprox,
                      discount);
+    times_in(iter, 3) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
 
     // joint the existent parameters with
     // the simulated new ones
@@ -817,6 +862,7 @@ Rcpp::List cICS_mv(arma::mat data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    in_start_s = clock();
 
     // update cluster allocations
     clust_update_ICS_mv(data,
@@ -824,6 +870,7 @@ Rcpp::List cICS_mv(arma::mat data,
                         s2join,
                         probjoin,
                         clust);
+    times_in(iter, 4) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
 
     mu = mujoin;
     s2 = s2join;
@@ -893,6 +940,8 @@ Rcpp::List cICS_mv(arma::mat data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
+  // TEMP TEMP TEMP
+  resu["wvals"] = times_in;
   return resu;
 }
 
@@ -1385,3 +1434,240 @@ Rcpp::List cICS_mv_MKR(arma::vec y,
   }
   return resu;
 }
+
+/*----------------------------------------------------------------------
+ *
+ * MULTIVARIATE
+ * MIXTURE OF KERNEL REGRESSION - LOCATION
+ * importance conditional sampler function
+ *
+ *----------------------------------------------------------------------
+ */
+
+//' @export
+//' @name cICS_mv_MKR_L
+//' @title C++ function to estimate Pitman-Yor multivariate mixtures via importance conditional sampler - PRODUCT KERNEL
+//' @keywords internal
+//'
+//' @param data a matrix of observations
+//' @param grid matrix of points to evaluate the density
+//' @param niter number of iterations
+//' @param nburn number of burn-in iterations
+//' @param m0 expectation of location component
+//' @param k0 vector, scale parameters for the location component
+//' @param a0 vector, parameters of scale component
+//' @param b0 vector, parameters of scale component
+//' @param m1 means of hyperdistribution of m0
+//' @param s21 variances of hyperdistribution of m0
+//' @param a1 shape parameters of hyperdistribution of b0
+//' @param b1 rate parameters of hyperdistribution of b0
+//' @param strength strength parameter
+//' @param napprox number of approximating values
+//' @param nupd number of iterations to show current updating
+//' @param out_param if TRUE, return also the location and scale paramteres lists
+//' @param out_dens if TRUE, return also the estimated density (default TRUE)
+//' @param discount second parameter of PY
+//' @param print_message print the status
+//' @param light_dens if TRUE return only the posterior mean of the density
+//' @param hyper, if TRUE use hyperpriors, default TRUE
+//
+
+//[[Rcpp::export]]
+Rcpp::List cICS_mv_MKR_L(arma::vec y,
+                         arma::mat covs,
+                         arma::vec grid_response,
+                         arma::mat grid_covs,
+                         int niter,
+                         int nburn,
+                         arma::vec beta0,
+                         arma::mat Sb0,
+                         double a0,
+                         double b0,
+                         arma::vec beta1,
+                         double k1,
+                         double sb1,
+                         arma::mat Sb1,
+                         double strength,
+                         int napprox,
+                         int nupd = 0,
+                         bool out_param = 0,
+                         bool out_dens = 1,
+                         double discount = 0,
+                         bool print_message = 1,
+                         bool light_dens = 1,
+                         bool hyper = 1){
+
+  if(nupd == 0){
+    nupd = (int) (niter / 10);
+  }
+  int n = covs.n_rows;
+  int d = covs.n_cols;
+
+  // initialize results objects
+  arma::mat result_clust(niter - nburn, n);
+  arma::mat result_beta(niter - nburn, d);
+  std::list<arma::mat> result_beta_all;
+  arma::vec result_sigma2(niter - nburn);
+  std::list<arma::vec> result_probs;
+
+  // initialize the result densities objects (response and covs)
+  arma::cube result_dens(grid_response.n_rows, grid_covs.n_rows, 1);
+  if(!light_dens){
+    result_dens.resize(grid_response.n_rows, grid_covs.n_rows, niter - nburn);
+  }
+  result_dens.fill(0);
+
+  // initialize required object inside the loop
+  arma::vec clust(n);
+  arma::mat beta(1,d);
+  double sigma2;
+  arma::mat betatemp(1,d);
+  arma::vec freqtemp(1);
+  arma::vec ptilde;
+  arma::mat dens(grid_response.n_rows, grid_covs.n_rows);
+  arma::vec probjoin;
+  arma::mat betajoin;
+
+  // fill the initialized quantity
+  clust.fill(0);
+  beta.row(0) = arma::trans(beta0);
+  sigma2 = b0 / (a0 - 1);
+  betatemp.fill(0);
+  freqtemp.fill(1);
+  dens.fill(0);
+
+  // time quantities
+  int start_s = clock();
+  int current_s;
+
+  // strarting loop
+  for(arma::uword iter = 0; iter < niter; iter++){
+
+    // acceleration step
+    accelerate_ICS_mv_MRK_L(y,
+                            covs,
+                            beta,
+                            sigma2,
+                            clust,
+                            beta0,
+                            Sb0,
+                            a0,
+                            b0);
+
+    if(hyper){
+      hyper_accelerate_ICS_mv_MRK_L(y,
+                                    covs,
+                                    clust,
+                                    beta,
+                                    beta0,
+                                    Sb0,
+                                    a0,
+                                    b0,
+                                    beta1,
+                                    k1,
+                                    sb1,
+                                    Sb1);
+    }
+
+    // sample the probability vector
+    // from Dirichlet distribution
+    arma::vec temp_freq = freq_vec(clust);
+    ptilde = rdirich_mass(temp_freq - discount, strength + discount * temp_freq.n_elem);
+
+    // simulate the required values
+    simu_trunc_PY_mv_MRK_L(betatemp,
+                           freqtemp,
+                           strength,
+                           beta0,
+                           Sb0,
+                           napprox,
+                           discount);
+
+    // joint the existent parameters with
+    // the simulated new ones
+    betajoin = arma::join_cols(beta, betatemp);
+
+    int nkpt = ptilde.n_elem;
+    arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
+    probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
+                               ptilde[nkpt - 1] * freqtemp / napprox);
+
+    // update cluster allocations
+    clust_update_ICS_mv_MRK_L(y,
+                              covs,
+                              betajoin,
+                              sigma2,
+                              probjoin,
+                              clust);
+
+    beta = betajoin;
+
+    // if the burn-in phase is complete
+    if(iter >= nburn){
+      result_probs.push_back(probjoin);
+
+      if(out_dens){
+        dens = eval_density_mv_MKR_L(grid_covs,
+                                     grid_response,
+                                     betajoin,
+                                     sigma2,
+                                     probjoin);
+
+        if(light_dens){
+          result_dens.slice(0) += dens;
+        } else {
+          result_dens.slice(iter - nburn) = dens;
+        }
+      }
+    }
+
+    // clean parameter objects
+    para_clean_ICS_mv_MRK_L(beta,
+                            clust);
+
+    // if the burn-in phase is complete
+    if(iter >= nburn){
+      result_beta_all.push_back(beta);
+      result_sigma2(iter - nburn) = sigma2;
+      result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+    }
+
+    if(print_message){
+      // print the current completed work
+      if((iter + 1) % nupd == 0){
+        current_s = clock();
+        Rcpp::Rcout << "Completed:\t" << (iter + 1) << "/" << niter << " - in " <<
+          double(current_s-start_s)/CLOCKS_PER_SEC << " sec\n";
+      }
+    }
+    Rcpp::checkUserInterrupt();
+  }
+
+  int end_s = clock();
+  if(print_message){
+    Rcpp::Rcout << "\n" << "Estimation done in " << double(end_s-start_s)/CLOCKS_PER_SEC << " seconds\n";
+  }
+
+  Rcpp::List resu;
+  if(out_param){
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["beta"]   = result_beta_all;
+    resu["sigma2"] = result_sigma2;
+    resu["probs"]  = result_probs;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  } else {
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  }
+  return resu;
+}
