@@ -859,9 +859,14 @@ Rcpp::List cSLI_mv(arma::mat data,
   int start_s = clock();
   int current_s;
 
+  // TEMP TEMP TEMP TEMP
+  arma::mat times_in(niter, 6);
+
   // strarting loop
   for(arma::uword iter = 0; iter < niter; iter++){
 
+    int in_start_s = clock();
+
     // update the parameters
     accelerate_SLI_PY_mv(data,
                          mu,
@@ -875,6 +880,8 @@ Rcpp::List cSLI_mv(arma::mat data,
                          n0,
                          mass,
                          sigma_PY);
+    times_in(iter, 0) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     if(hyper){
       hyper_accelerate_SLI_mv_LS(mu,
@@ -891,6 +898,8 @@ Rcpp::List cSLI_mv(arma::mat data,
                                  theta1,
                                  Theta1);
     }
+    times_in(iter, 1) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+    in_start_s = clock();
 
     if(indep){
       // update the stick breaking weights
@@ -923,11 +932,14 @@ Rcpp::List cSLI_mv(arma::mat data,
                                   w,
                                   xi,
                                   u);
+
     } else {
       // update the stick breaking weights
       update_u_SLI(clust,
                    w,
                    u);
+      times_in(iter, 2) = double(clock() - in_start_s ) / CLOCKS_PER_SEC ;
+      in_start_s = clock();
 
       // extend the stick breaking representation
       grow_param_SLI_PY_mv(mu,
@@ -942,6 +954,8 @@ Rcpp::List cSLI_mv(arma::mat data,
                            mass,
                            n,
                            sigma_PY);
+      times_in(iter, 3) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+      in_start_s = clock();
 
       // update the allocation
       update_cluster_SLI_mv(data,
@@ -950,6 +964,8 @@ Rcpp::List cSLI_mv(arma::mat data,
                             clust,
                             w,
                             u);
+      times_in(iter, 4) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+
     }
 
     // save the results
@@ -1019,6 +1035,8 @@ Rcpp::List cSLI_mv(arma::mat data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
+  // TEMP TEMP TEMP
+  resu["wvals"] = times_in;
   return resu;
 }
 
@@ -1570,3 +1588,265 @@ Rcpp::List cSLI_mv_MKR(arma::vec y,
   }
   return resu;
 }
+
+/*----------------------------------------------------------------------
+ *
+ * MULTIVARIATE
+ * MIXTURE OF KERNEL REGRESSION
+ * slice sampler function
+ *
+ *----------------------------------------------------------------------
+ */
+
+//' @export
+//' @name cSLI_mv_MKR_L
+//' @title C++ function to estimate Pitman-Yor multivariate mixtures via slice sampler - PRODUCT KERNEL
+//' @keywords internal
+//'
+//' @param data a matrix of observations
+//' @param grid matrix of points to evaluate the density
+//' @param niter number of iterations
+//' @param nburn number of burn-in iterations
+//' @param m0 expectation of location component
+//' @param k0 vector, scale parameters for the location component
+//' @param a0 vector, parameters of scale component
+//' @param b0 vector, parameters of scale component
+//' @param m1 means of hyperdistribution of m0
+//' @param s21 variances of hyperdistribution of m0
+//' @param a1 shape parameters of hyperdistribution of b0
+//' @param b1 rate parameters of hyperdistribution of b0
+//' @param strength strength parameter
+//' @param napprox number of approximating values
+//' @param nupd number of iterations to show current updating
+//' @param out_param if TRUE, return also the location and scale paramteres lists
+//' @param out_dens if TRUE, return also the estimated density (default TRUE)
+//' @param discount second parameter of PY
+//' @param print_message print the status
+//' @param light_dens if TRUE return only the posterior mean of the density
+//' @param hyper, if TRUE use hyperpriors, default TRUE
+//
+
+//[[Rcpp::export]]
+Rcpp::List cSLI_mv_MKR_L(arma::vec y,
+                         arma::mat covs,
+                         arma::vec grid_response,
+                         arma::mat grid_covs,
+                         int niter,
+                         int nburn,
+                         arma::vec beta0,
+                         arma::mat Sb0,
+                         double a0,
+                         double b0,
+                         arma::vec beta1,
+                         double k1,
+                         double sb1,
+                         arma::mat Sb1,
+                         double strength,
+                         double param_seq_one,
+                         double param_seq_two,
+                         int nupd = 0,
+                         bool out_param = 0,
+                         bool out_dens = 1,
+                         double discount = 0,
+                         bool print_message = 1,
+                         bool light_dens = 1,
+                         bool hyper = 1,
+                         bool indep = true){
+  arma::vec xi(1);
+  if(indep){
+    xi(0) = (1 - param_seq_two) / (1 + param_seq_one);
+  }
+
+  if(nupd == 0){
+    nupd = (int) (niter / 10);
+  }
+  int n = covs.n_rows;
+  int d = covs.n_cols;
+
+  // initialize results objects
+  arma::mat result_clust(niter - nburn, n);
+  arma::mat result_beta(niter - nburn, d);
+  std::list<arma::mat> result_beta_all;
+  arma::vec result_sigma2(niter - nburn);
+  std::list<arma::vec> result_probs;
+
+  // initialize the result densities objects (response and covs)
+  arma::cube result_dens(grid_response.n_rows, grid_covs.n_rows, 1);
+  if(!light_dens){
+    result_dens.resize(grid_response.n_rows, grid_covs.n_rows, niter - nburn);
+  }
+  result_dens.fill(0);
+
+  // initialize required object inside the loop
+  arma::vec clust(n);
+  arma::mat beta(1,d);
+  double sigma2;
+  arma::mat dens(grid_response.n_rows, grid_covs.n_rows);
+  arma::vec v(1);
+  arma::vec w(1);
+  arma::vec u(n, arma::fill::randu);
+
+  // fill the initialized quantity
+  clust.fill(0);
+  beta.row(0) = arma::trans(beta0);
+  sigma2 = b0 / (a0 - 1);
+
+  // time quantities
+  int start_s = clock();
+  int current_s;
+
+  // strarting loop
+  for(arma::uword iter = 0; iter < niter; iter++){
+
+    // acceleration step
+    accelerate_SLI_mv_MRK_L(y,
+                            covs,
+                            beta,
+                            sigma2,
+                            v,
+                            w,
+                            clust,
+                            beta0,
+                            Sb0,
+                            a0,
+                            b0,
+                            strength,
+                            discount);
+
+    if(hyper){
+      hyper_accelerate_SLI_mv_MRK_L(y,
+                                    covs,
+                                    clust,
+                                    beta,
+                                    beta0,
+                                    Sb0,
+                                    beta1,
+                                    k1,
+                                    sb1,
+                                    Sb1);
+
+    }
+
+    if(indep){
+      // update the stick breaking weights
+      update_u_SLI(clust,
+                   xi,
+                   u);
+
+      // extend the stick breaking representation
+      grow_param_indep_SLI_PY_mv_MRK_L(beta,
+                                       v,
+                                       w,
+                                       xi,
+                                       u,
+                                       beta0,
+                                       Sb0,
+                                       strength,
+                                       n,
+                                       discount,
+                                       param_seq_one,
+                                       param_seq_two);
+
+      // update the allocation
+      update_cluster_indep_SLI_mv_MRK_L(y,
+                                        covs,
+                                        beta,
+                                        sigma2,
+                                        clust,
+                                        w,
+                                        xi,
+                                        u);
+    } else {
+      // update the stick breaking weights
+      update_u_SLI(clust,
+                   w,
+                   u);
+
+      // extend the stick breaking representation
+      grow_param_SLI_PY_mv_MRK_L(beta,
+                                 v,
+                                 w,
+                                 u,
+                                 beta0,
+                                 Sb0,
+                                 strength,
+                                 n,
+                                 discount);
+
+      // update the allocation
+      update_cluster_SLI_mv_MRK_L(y,
+                                  covs,
+                                  beta,
+                                  sigma2,
+                                  clust,
+                                  w,
+                                  u);
+    }
+
+    // if the burn-in phase is complete
+    if(iter >= nburn){
+      result_probs.push_back(w);
+      result_beta_all.push_back(beta);
+      result_sigma2(iter - nburn) = sigma2;
+      result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+
+      if(out_dens){
+        dens = eval_density_mv_MKR_L(grid_covs,
+                                     grid_response,
+                                     beta,
+                                     sigma2,
+                                     w);
+
+        if(light_dens){
+          result_dens.slice(0) += dens;
+        } else {
+          result_dens.slice(iter - nburn) = dens;
+        }
+      }
+    }
+
+    beta.resize(max(clust) + 1, d);
+    w.resize(max(clust) + 1);
+    v.resize(max(clust) + 1);
+    xi.resize(max(clust) + 1);
+
+
+    if(print_message){
+      // print the current completed work
+      if((iter + 1) % nupd == 0){
+        current_s = clock();
+        Rcpp::Rcout << "Completed:\t" << (iter + 1) << "/" << niter << " - in " <<
+          double(current_s-start_s)/CLOCKS_PER_SEC << " sec\n";
+      }
+    }
+    Rcpp::checkUserInterrupt();
+  }
+
+  int end_s = clock();
+  if(print_message){
+    Rcpp::Rcout << "\n" << "Estimation done in " << double(end_s-start_s)/CLOCKS_PER_SEC << " seconds\n";
+  }
+
+  Rcpp::List resu;
+  if(out_param){
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["beta"]   = result_beta_all;
+    resu["sigma2"] = result_sigma2;
+    resu["probs"]  = result_probs;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  } else {
+    if(light_dens){
+      resu["dens"]   = result_dens / (niter - nburn);
+    } else {
+      resu["dens"]   = result_dens;
+    }
+    resu["clust"]  = result_clust;
+    resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
+  }
+  return resu;
+}
